/** Copyright (C) 2017 by Bundesanstalt für Gewässerkunde
 * Software engineering by
 *  Björnsen Beratende Ingenieure GmbH
 *  Dr. Schumacher Ingenieurbüro für Wasser und Umwelt
 *
 * This file is Free Software under the GNU AGPL (>=v3)
 * and comes with ABSOLUTELY NO WARRANTY! Check out the
 * documentation coming with Dive4Elements River for details.
 */
package org.dive4elements.river.artifacts.uinfo.salix;

import java.text.NumberFormat;
import java.util.Collection;
import java.util.List;

import org.dive4elements.river.artifacts.common.ExportContextCSV;
import org.dive4elements.river.artifacts.common.GeneralResultType;
import org.dive4elements.river.artifacts.common.IExportContext;
import org.dive4elements.river.artifacts.common.ResultRow;
import org.dive4elements.river.artifacts.sinfo.util.RiverInfo;
import org.dive4elements.river.artifacts.uinfo.commons.UInfoResultType;

import gnu.trove.TDoubleArrayList;

/**
 * @author Domenico Nardi Tironi
 */
final class SalixLineCalculationRegionalResult extends SalixLineCalculationNoScenarioResult {

    private static final long serialVersionUID = 1L;

    private final String[] scenarioLabels;

    public SalixLineCalculationRegionalResult(final String label, final String[] scenarioLabels, final Collection<ResultRow> rows) {
        super(label, rows);
        this.scenarioLabels = scenarioLabels;
    }

    public final int getScenarioCount() {
        return (this.scenarioLabels != null) ? this.scenarioLabels.length : 0;
    }

    public String getScenarioLabel(final int i) {
        if ((this.scenarioLabels != null) && (i < this.scenarioLabels.length))
            return this.scenarioLabels[i];
        else
            return "";
    }

    @Override
    public void writeCSVHeader(final ExportContextCSV exportContextCSV, final RiverInfo river) {

        final Collection<String> header = super.writeNoScenarioHeader(exportContextCSV);

        for (int i = 1; i <= getScenarioCount(); i++) {
            header.add(exportContextCSV.msg(SalixScenario.getScenarioValueHeader()));
            header.add(exportContextCSV.msg(SalixScenario.getDwsplHeader()));
        }
        exportContextCSV.writeCSVLine(header.toArray(new String[header.size()]));
    }

    @Override
    protected void writeCSVResultMetadata(final ExportContextCSV exportContextCSV) {
        writeRegionalEffectsCSVMetadata(exportContextCSV);
        exportContextCSV.writeBlankLine();

    }

    @Override
    protected String[] formatRow(final IExportContext context, final ResultRow row) {

        final Collection<String> lines = super.getNoScenarioFormat(row, context);

        if (context instanceof ExportContextCSV) { // enum gespart - pdf-export hat ja nur die drei spalten
            final NumberFormat formatter = ((ExportContextCSV) context).getSalixScenFormatter();
            final List<SalixScenario> scenList = (List<SalixScenario>) row.getValue(UInfoResultType.customMultiRowColSalixRegionalValue_Dwspl);

            for (final SalixScenario scenario : scenList) {
                // scenario unused; index wurde ja verworfen, stattdessen 2 spalten
                lines.add(scenario.getSalixValueFormatted(formatter));
                lines.add(scenario.getDwsplFormatted());
            }
        }
        return lines.toArray(new String[lines.size()]);
    }

    private void writeRegionalEffectsCSVMetadata(final ExportContextCSV exportContextCSV) {
        final String main = "uinfo.export.csv.meta.header.salix.regional";
        // "##Regional wirkende Eingriffe"
        exportContextCSV.writeCSVMetaEntry(main);
        writeRegionalCommonCSVMeta(exportContextCSV);
    }

    /**
     * Gets a longitudinal section of W, Q, or flood duration of one of the waterlevels
     */
    public final double[][] getScenarioPoints(final int dataIndex) {

        final Collection<ResultRow> rows = getRows();

        final TDoubleArrayList xPoints = new TDoubleArrayList(rows.size());
        final TDoubleArrayList yPoints = new TDoubleArrayList(rows.size());

        for (final ResultRow row : rows) {

            final double station = row.getDoubleValue(GeneralResultType.station);

            final List<SalixScenario> scenarios = (List<SalixScenario>) row.getValue(UInfoResultType.customMultiRowColSalixRegionalValue_Dwspl);
            if (dataIndex <= scenarios.size() - 1) {
                final double value = scenarios.get(dataIndex).getSalixValue();
                xPoints.add(station);
                yPoints.add(value);
            }
        }

        return new double[][] { xPoints.toNativeArray(), yPoints.toNativeArray() };
    }

    // TODO: move to SupraRegionalEffectsResult-Class; remove here
    // private void writeExtendedRegionalEffectsCSVMetadata(final ExportContextCSV exportContextCSV) {
    // final String main = "uinfo.export.csv.meta.header.salix.regionalextended";
    // // "##Überregional wirkende Eingriffe"
    // exportContextCSV.writeCSVMetaEntry(main);
    //
    // writeRegionalCommonCSVMeta(exportContextCSV);
    // }

    // TODO: move to historical class and delete here
    // private void writeHistoricalViewCSVMetadata(final ExportContextCSV exportContextCSV) {
    // final String main = "uinfo.export.csv.meta.header.salix.historical";
    // // "##Historische Betrachtung"
    // exportContextCSV.writeCSVMetaEntry(main);
    //
    // final String mainSub = "uinfo.export.csv.meta.header.salix";
    // // "# Szenariotyp: "
    // exportContextCSV.writeCSVMetaEntry(mainSub + ".szenariotyp");
    // // "# Teilabschnitt: "
    // exportContextCSV.writeCSVMetaEntry(mainSub + ".teilabschnitt");
    //
    // // "# Art des Zeitraums: "
    // exportContextCSV.writeCSVMetaEntry(main + ".zeitart");
    // // "# Historischer Zeitpunkt: "
    // exportContextCSV.writeCSVMetaEntry(main + ".zeitpunkt");
    // }
}